@@ -25,16 +25,19 @@ import termios
 import struct
 import subprocess
 import logging
+from pathlib import Path
 import pyte
-from host_ip import get_host_ip
+from . import host_ip
+from flask_swagger_ui import get_swaggerui_blueprint  # new import
+import yaml
 
 if os.environ.get('PYWEBEXEC_LDAP_SERVER'):
     from ldap3 import Server, Connection, ALL, SIMPLE, SUBTREE, Tls
 
 app = Flask(__name__)
 app.secret_key = os.urandom(24)  # Secret key for session management
+app.json.sort_keys = False
 app.config['SESSION_COOKIE_SAMESITE'] = 'Lax'  # Add SameSite attribute to session cookies
-app.config['SESSION_COOKIE_SECURE'] = True
 app.config['SESSION_COOKIE_HTTPONLY'] = True
 auth = HTTPBasicAuth()
 
@@ -45,6 +48,8 @@ app.config['LDAP_BASE_DN'] = os.environ.get('PYWEBEXEC_LDAP_BASE_DN')
 app.config['LDAP_BIND_DN'] = os.environ.get('PYWEBEXEC_LDAP_BIND_DN')
 app.config['LDAP_BIND_PASSWORD'] = os.environ.get('PYWEBEXEC_LDAP_BIND_PASSWORD')
 
+app.config["JSON_SORT_KEYS"] = False
+
 # Get the Gunicorn error logger
 gunicorn_logger = logging.getLogger('gunicorn.error')
 app.logger.handlers = gunicorn_logger.handlers
@@ -164,12 +169,22 @@ def strip_ansi_control_chars(text):
 def decode_line(line: bytes) -> str:
     """try decode line exception on binary"""
     try:
-        return get_visible_output(line.decode())
+        return get_visible_line(line.decode())
+    except UnicodeDecodeError:
+        return ""
+
+def get_visible_output(output, cols, rows):
+    """pyte vt100 render to get last line"""
+    try:
+        screen = pyte.Screen(cols, rows)
+        stream = pyte.Stream(screen)
+        stream.feed(output)
+        return "\n".join(screen.display).strip()
     except UnicodeDecodeError:
         return ""
 
 
-def get_visible_output(line, cols, rows):
+def get_visible_line(line, cols, rows):
     """pyte vt100 render to get last line"""
     try:
         screen = pyte.Screen(cols, rows)
@@ -194,7 +209,7 @@ def get_last_line(file_path, cols=None, rows=None, maxsize=2048):
             fd.seek(-maxsize, os.SEEK_END)
         except OSError:
             fd.seek(0)
-        return get_visible_output(fd.read(), cols, rows)
+        return get_visible_line(fd.read(), cols, rows)
 
 
 def start_gunicorn(daemonized=False, baselog=None):
@@ -303,10 +318,10 @@ def print_urls(command_id=None):
     if token:
         url_params = f"?token={token}"
     if command_id:
-        print(f"web popup: {protocol}://{hostname}:{args.port}/dopopup/{command_id}{url_params}", flush=True)
-        print(f"web popup: {protocol}://{ip}:{args.port}/dopopup/{command_id}{url_params}", flush=True)
-        print(f"raw output: {protocol}://{hostname}:{args.port}/command_output_raw/{command_id}{url_params}", flush=True)
-        print(f"raw output: {protocol}://{ip}:{args.port}/command_output_raw/{command_id}{url_params}", flush=True)
+        print(f"web popup: {protocol}://{hostname}:{args.port}/commands/{command_id}/dopopup{url_params}", flush=True)
+        print(f"web popup: {protocol}://{ip}:{args.port}/commands/{command_id}/dopopup{url_params}", flush=True)
+        print(f"raw output: {protocol}://{hostname}:{args.port}/commands/{command_id}/output_raw{url_params}", flush=True)
+        print(f"raw output: {protocol}://{ip}:{args.port}/commands/{command_id}/output_raw{url_params}", flush=True)
     else:
         print(f"web commands: {protocol}://{hostname}:{args.port}{url_params}", flush=True)
         print(f"web commands: {protocol}://{ip}:{args.port}{url_params}", flush=True)
@@ -365,7 +380,7 @@ def parseargs():
         COMMAND_STATUS_DIR = f"{os.getcwd()}/{COMMAND_STATUS_DIR}"
         sys.exit(start_term())
 
-    (hostname, ip) = get_host_ip(args.listen)
+    (hostname, ip) = host_ip.get_host_ip(args.listen)
 
     if args.tokenurl:
         token = os.environ.get("PYWEBEXEC_TOKEN", token_urlsafe())
@@ -403,37 +418,42 @@ def get_output_file_path(command_id):
 
 def update_command_status(command_id, updates):
     status_file_path = get_status_file_path(command_id)
-    status = read_command_status(command_id) or {}
+    status = read_command_status(command_id)
     status.update(updates)
+    status = status.copy()
     if status.get('status') != 'running':
         output_file_path = get_output_file_path(command_id)
         if os.path.exists(output_file_path):
             status['last_output_line'] = get_last_line(output_file_path, status.get('cols'), status.get('rows'))
-    status_cache[command_id] = status
+    if 'last_read' in status:
+        del status['last_read']
     with open(status_file_path, 'w') as f:
         json.dump(status, f)
+    os.sync()
+    status_cache[command_id] = status
 
 
 def read_command_status(command_id):
     # Return cached status if available
-    status_data = {}
-    if command_id in status_cache:
-        status_data = status_cache[command_id]
+    global status_cache
+    if not command_id in status_cache:
+        status_cache[command_id] = {}
+    status_data = status_cache[command_id]
     status = status_data.get('status')
     if status and status != "running":
         return status_data
-    if command_id in status_cache and status_cache.get('last_read',0)>datetime.now().timestamp()-0.5:
+    if status_data.get('last_read',0)>datetime.now().timestamp()-0.5:
         return status_data
     status_file_path = get_status_file_path(command_id)
     if not os.path.exists(status_file_path):
-        return None
+        return status_data
     with open(status_file_path, 'r') as f:
         try:
             status_data.update(json.load(f))
         except json.JSONDecodeError:
-            return None
+            return status_data
     status_data['last_read'] = datetime.now().timestamp()
-    status_cache[command_id] = status_data
+    #status_cache[command_id] = status_data
     return status_data
 
 def sigwinch_passthrough(sig, data):
@@ -465,7 +485,6 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
     log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}')
     start_time = datetime.now(timezone.utc).isoformat()
     update_command_status(command_id, {
-        'status': 'running',
         'command': command,
         'params': params,
         'start_time': start_time,
@@ -492,7 +511,6 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
                     'status': 'aborted',
                     'end_time': end_time,
                     'exit_code': exit_code,
-                    'user': user
                 })
                 log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: command aborted')
             else:
@@ -502,7 +520,6 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
                         'status': 'success',
                         'end_time': end_time,
                         'exit_code': exit_code,
-                        'user': user
                     })
                     log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: completed successfully')
                 else:
@@ -510,7 +527,6 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
                         'status': 'failed',
                         'end_time': end_time,
                         'exit_code': exit_code,
-                        'user': user
                     })
                     log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}: exit code {exit_code}')
 
@@ -520,7 +536,6 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
             'status': 'failed',
             'end_time': end_time,
             'exit_code': 1,
-            'user': user
         })
         with open(get_output_file_path(command_id), 'a') as output_file:
             output_file.write(str(e))
@@ -554,7 +569,6 @@ def read_commands():
                                 'last_update': datetime.now().timestamp(),
                                 'last_output_line': get_last_line(output_file_path, status.get('cols'), status.get('rows')),
                             })
-                            status_cache[command_id] = status
                 commands.append({
                     'command_id': command_id,
                     'status': status.get('status'),
@@ -590,7 +604,23 @@ def check_processes():
                         'exit_code': -1,
                     })
 
-parseargs()
+args = parseargs()
+if args.cert:
+    app.config['SESSION_COOKIE_SECURE'] = True
+app.config['TITLE'] = f"{args.title} API"
+
+# Register Swagger UI blueprint with safe token included in API_URL
+SWAGGER_URL = '/v0/documentation'
+API_URL = '/swagger.yaml'
+swaggerui_blueprint = get_swaggerui_blueprint(
+    SWAGGER_URL,
+    API_URL,
+    config={
+        'app_name': f"{args.title} API",
+        'layout': 'BaseLayout'
+    }
+)
+app.register_blueprint(swaggerui_blueprint, url_prefix=SWAGGER_URL)
 
 def log_info(fromip, user, message):
     app.logger.info(f"{user} {fromip}: {message}")
@@ -601,7 +631,30 @@ def log_error(fromip, user, message):
 def log_request(message):
     log_info(request.remote_addr, session.get('username', '-'), message)
 
-@app.route('/stop_command/<command_id>', methods=['POST'])
+def get_executable(cmd):
+    if os.path.isfile(cmd) and os.access(cmd, os.X_OK):
+        help_file = f"{cmd}.help"
+        help_text = ""
+        if os.path.exists(help_file) and os.path.isfile(help_file):
+            with open(help_file, 'r') as hf:
+                help_text = hf.read()
+        schema_file = f"{cmd}.schema.yaml"
+        schema = None
+        if os.path.exists(schema_file):
+            with open(schema_file, 'r') as sf:
+                schema = yaml.safe_load(sf)
+        return {"command": cmd, "help": help_text, "schema": schema}
+    return None
+
+def get_executables():
+    executables_list = []
+    for f in os.listdir('.'):
+            exe = get_executable(f)
+            if exe:
+                executables_list.append(exe)
+    return sorted(executables_list, key=lambda x: x["command"])
+
+@app.route('/commands/<command_id>/stop', methods=['PATCH'])
 def stop_command(command_id):
     log_request(f"stop_command {command_id}")
     status = read_command_status(command_id)
@@ -611,7 +664,6 @@ def stop_command(command_id):
     pid = status['pid']
     end_time = datetime.now(timezone.utc).isoformat()
     try:
-        #update_command_status(command_id, 'aborted', end_time=end_time, exit_code=-15)
         os.killpg(os.getpgid(pid), 15)  # Send SIGTERM to the process group
         return jsonify({'message': 'Command aborted'})
     except Exception as e:
@@ -625,10 +677,10 @@ def stop_command(command_id):
 
 @app.before_request
 def check_authentication():
-    # Check for token in URL if TOKEN_URL is set
     token = app.config.get('TOKEN_URL')
     if token and request.endpoint not in ['login', 'static']:
-        if request.args.get('token') == token:
+        if request.args.get('token') == token or session.get('token') == token:
+            session['token'] = token
             return
         return jsonify({'error': 'Forbidden'}), 403
 
@@ -686,7 +738,7 @@ def verify_ldap(username, password):
         print(f"LDAP authentication failed: {e}")
         return False
 
-@app.route('/run_command', methods=['POST'])
+@app.route('/commands', methods=['POST'])
 def run_command_endpoint():
     data = request.json
     command = data.get('command')
@@ -720,24 +772,98 @@ def run_command_endpoint():
         'from': request.remote_addr,
     })
 
+    Path(get_output_file_path(command_id)).touch()
     # Run the command in a separate thread
     thread = threading.Thread(target=run_command, args=(request.remote_addr, user, command_path, params, command_id, rows, cols))
     thread.start()
 
     return jsonify({'message': 'Command is running', 'command_id': command_id})
 
-@app.route('/command_status/<command_id>', methods=['GET'])
+@app.route('/commands/exposed', methods=['GET'])
+def list_executables():
+    executables_list = get_executables()
+    return jsonify({"commands": executables_list})
+
+
+@app.route('/commands/<cmd>', methods=['POST'])
+def run_dynamic_command(cmd):
+    # Validate that 'cmd' is an executable in the current directory
+    cmd_path = os.path.join(".", os.path.basename(cmd))
+    if not os.path.isfile(cmd_path) or not os.access(cmd_path, os.X_OK):
+        return jsonify({'error': 'Command not found or not executable'}), 400
+    try:
+        data = request.json
+    except Exception as e:
+        data = {}
+
+    # Convert received parameters to exec args.
+    # If schema defined for each para, value in data:
+    # - if value is True → --<key>
+    # - if value is a string → --<param> value
+    # - if value is an array → --<param> value1 value2 ...
+    # - if value is False → "" (omit)
+    # schema_options:
+    #   separator: " " (default) or "=" is the separator between --param and value
+    #   noprefix_params: ["param1", "param2"] or ["*"] to omit --param prefix
+    #   convert_params: {"param1": "param2"} to convert param1 to param2
+    exe = get_executable(cmd)
+    separator = exe.get("schema", {}).get("schema_options", {}).get("separator", " ")
+    noprefix = exe.get("schema", {}).get("schema_options", {}).get("noprefix_params", {})
+    convert_params = exe.get("schema", {}).get("schema_options", {}).get("convert_params", {})
+    if data.get('params') and isinstance(data['params'], dict):
+        params = ""
+        for param, value in data['params'].items():
+            if value is None:
+                continue
+            prefix = ""
+            if param in convert_params:
+                param = convert_params[param]
+                prefix = param
+                if param in ['--', '', None]:
+                    separator = ' '
+            elif "*" in noprefix or param in noprefix:
+                separator = ""
+            else:
+                prefix = f"--{param}"
+            if isinstance(value, bool):
+                if value:
+                    params += f"{prefix} "
+                continue
+            params += f"{prefix}{separator}"
+            if isinstance(value, list):
+                params += " ".join(value)
+            else:
+                params += str(value)
+            params += " "
+    else:
+        params = data.get("params", [])
+    if isinstance(params, str):
+        params = [params]
+    try:
+        params = shlex.split(' '.join(params)) if isinstance(params, list) else []
+    except Exception as e:
+        return jsonify({'error': str(e)}), 400
+    rows = data.get('rows', tty_rows) or tty_rows
+    cols = data.get('cols', tty_cols) or tty_cols
+    user = session.get('username', '-')
+    command_id = str(uuid.uuid4())
+    update_command_status(command_id, {
+        'status': 'running',
+        'command': cmd,
+        'params': params,
+        'user': user,
+        'from': request.remote_addr,
+    })
+    Path(get_output_file_path(command_id)).touch()
+    thread = threading.Thread(target=run_command, args=(request.remote_addr, user, cmd_path, params, command_id, rows, cols))
+    thread.start()
+    return jsonify({'message': 'Command is running', 'command_id': command_id})
+
+@app.route('/commands/<command_id>', methods=['GET'])
 def get_command_status(command_id):
     status = read_command_status(command_id)
     if not status:
         return jsonify({'error': 'Invalid command_id'}), 404
-
-    # output_file_path = get_output_file_path(command_id)
-    # if os.path.exists(output_file_path):
-    #     with open(output_file_path, 'r') as output_file:
-    #         output = output_file.read()
-    #     status['output'] = output
-
     return jsonify(status)
 
 @app.route('/')
@@ -746,22 +872,27 @@ def index():
 
 @app.route('/commands', methods=['GET'])
 def list_commands():
-    # Sort commands by start_time in descending order
     commands = read_commands()
     commands.sort(key=lambda x: x['start_time'], reverse=True)
-    return jsonify(commands)
+    return jsonify({"commands": commands})
 
-@app.route('/command_output/<command_id>', methods=['GET'])
+@app.route('/commands/<command_id>/output', methods=['GET'])
 def get_command_output(command_id):
     offset = int(request.args.get('offset', 0))
     maxsize = int(request.args.get('maxsize', 10485760))
+    maxlines = int(request.args.get('maxlines', 5000))
     output_file_path = get_output_file_path(command_id)
     if os.path.exists(output_file_path):
-        with open(output_file_path, 'rb') as output_file:
-            output_file.seek(offset)
-            output = output_file.read().decode('utf-8', errors='replace')
-            new_offset = output_file.tell()
-        status_data = read_command_status(command_id) or {}
+        size = os.path.getsize(output_file_path)
+        if offset >= size:
+            output = ''
+            new_offset = offset
+        else:
+            with open(output_file_path, 'rb') as output_file:
+                output_file.seek(offset)
+                output = output_file.read().decode('utf-8', errors='replace')
+                new_offset = output_file.tell()
+        status_data = read_command_status(command_id)
         token = app.config.get("TOKEN_URL")
         token_param = f"&token={token}" if token else ""
         response = {
@@ -770,15 +901,15 @@ def get_command_output(command_id):
             'cols': status_data.get("cols"),
             'rows': status_data.get("rows"),
             'links': {
-                'next': f'{request.url_root}command_output/{command_id}?offset={new_offset}&maxsize={maxsize}{token_param}'
+                'next': f'{request.url_root}commands/{command_id}/output?offset={new_offset}&maxsize={maxsize}{token_param}'
             }
         }
         if request.headers.get('Accept') == 'text/plain':
-            return f"{output}\nstatus: {status_data.get('status')}", 200, {'Content-Type': 'text/plain'}
+            return f"{get_visible_output(output, status_data.get('cols'), maxlines)}\nstatus: {status_data.get('status')}", 200, {'Content-Type': 'text/plain'}
         return jsonify(response)
     return jsonify({'error': 'Invalid command_id'}), 404
 
-@app.route('/command_output_raw/<command_id>', methods=['GET'])
+@app.route('/commands/<command_id>/output_raw', methods=['GET'])
 def get_command_output_raw(command_id):
     offset = int(request.args.get('offset', 0))
     @stream_with_context
@@ -802,17 +933,12 @@ def get_command_output_raw(command_id):
             return
     return Response(generate(offset), content_type='text/plain')
 
-@app.route('/executables', methods=['GET'])
-def list_executables():
-    executables = [f for f in os.listdir('.') if os.path.isfile(f) and os.access(f, os.X_OK)]
-    executables.sort()  # Sort the list of executables alphabetically
-    return jsonify(executables)
 
-@app.route('/popup/<command_id>')
+@app.route('/commands/<command_id>/popup')
 def popup(command_id):
     return render_template('popup.html', command_id=command_id)
 
-@app.route('/dopopup/<command_id>')
+@app.route('/commands/<command_id>/dopopup')
 def do_popup(command_id):
     token = request.args.get('token', '')
     token_param = f'?token={token}' if token else ''
@@ -821,7 +947,7 @@ def do_popup(command_id):
     <head>
         <script type="text/javascript">
             window.onload = function() {{
-                window.open('/popup/{command_id}{token_param}', '_blank', 'width=1000,height=600');
+                window.open('/commands/{command_id}/popup{token_param}', '_blank', 'width=1000,height=600');
                 window.close();
             }};
         </script>
@@ -831,6 +957,73 @@ def do_popup(command_id):
     </html>
     """
 
+@app.route('/swagger.yaml')
+def swagger_yaml():
+    swagger_path = os.path.join(os.path.dirname(__file__), 'swagger.yaml')
+    try:
+        with open(swagger_path, 'r') as f:
+            swagger_spec_str = f.read()
+        swagger_spec = yaml.safe_load(swagger_spec_str)
+        # Update existing POST /commands enum if present
+        executables = get_executables()
+        post_cmd = swagger_spec.get('paths', {}).get('/commands', {}).get('post')
+        if post_cmd:
+            params_list = post_cmd.get('parameters', [])
+            for param in params_list:
+                if param.get('in') == 'body' and 'schema' in param:
+                    props = param['schema'].get('properties', {})
+                    if 'command' in props:
+                        props['command']['enum'] = [e['command'] for e in executables]
+        # Add dynamic paths for each 
+        # Add dynamic paths for each executable:
+        for exe in executables:
+            dynamic_path = "/commands/" + exe["command"]
+            cmd_schema = {
+                "type": "object",
+                "properties": {
+                    "params": {"type": "array", "items": {"type": "string"}, "default": []},
+                    "rows": {"type": "integer", "default": tty_rows},
+                    "cols": {"type": "integer", "default": tty_cols}
+                }
+            }
+            if exe["schema"]:
+                # if exec["schema"].get("schema_options"):
+                #     del exe["schema"]["schema_options"]
+                cmd_schema["properties"]["params"] = exe["schema"]
+            swagger_spec.setdefault("paths", {})[dynamic_path] = {
+                "post": {
+                    "summary": f"Run command {exe['command']}",
+                    "tags": ["run_commands"],
+                    "description": f"{exe['help']}",
+                    "consumes": ["application/json"],
+                    "produces": ["application/json"],
+                    "parameters": [
+                        {
+                            "in": "body",
+                            "name": "commandRequest",
+                            "schema": cmd_schema
+                        }
+                    ],
+                    "responses": {
+                        "200": {
+                            "description": "Command started",
+                            "schema": {
+                                "type": "object",
+                                "properties": {
+                                    "message": {"type": "string"},
+                                    "command_id": {"type": "string"}
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        swagger_spec['info']['title'] = app.config.get('TITLE', 'PyWebExec API')
+        swagger_spec_str = yaml.dump(swagger_spec)
+        return Response(swagger_spec_str, mimetype='application/yaml')
+    except Exception as e:
+        return Response(f"Error reading swagger spec: {e}", status=500)
+
 def main():
     global COMMAND_STATUS_DIR
     basef = f"{CONFDIR}/pywebexec_{args.listen}:{args.port}"
@@ -848,9 +1041,8 @@ def main():
         COMMAND_STATUS_DIR = f"{os.getcwd()}/{COMMAND_STATUS_DIR}"
         check_processes()
         sys.argv.remove("shareterm")
-        sys.argv[0] = PYWEBEXEC
         with open(basef + ".log", "a") as log:
-            pywebexec = subprocess.Popen([sys.executable] + sys.argv, stdout=log, stderr=log, bufsize=1)
+            pywebexec = subprocess.Popen(sys.argv, stdout=log, stderr=log, bufsize=1)
             print_urls(term_command_id)
             res = start_term()
             print("Stopping server")
