@@ -22,27 +22,37 @@ function initTerminal()
         cursorInactiveStyle: 'none',
         disableStdin: true,
         //convertEol: true,
-        fontFamily: '"Consolas NF", "Fira Code", monospace, "Powerline Extra Symbols", courier-new, courier',
-        fontSize: fontSize,
         scrollback: maxScrollback,
+        fontFamily: '"CommitMono Nerd Font Mono", "Fira Code", monospace, "Powerline Extra Symbols", courier-new, courier',
+        fontSize: fontSize,
+        letterSpacing: 0,
+        lineHeight: 1.1,
+        fontWeightBold: 500,
+        //fontWeight: 400,
+        screenReaderMode: true,
         theme: {
             background: '#111412',
+            foreground: '#d7d7d7',
+            white: '#d7d7d7',
             black: '#111412',
-            green: '#088a5b',
+            green: '#0d8657',
             blue: "#2760aa",
-            red: '#ba1611',
+            red: '#aa1e0f',
             yellow: "#cf8700",
             magenta: "#4c3d80",
-            cyan: "#00a7aa",
+            cyan: "#3b8ea7",
+            brightWhite: '#efefef',
             brightBlack: "#243C4F",
             brightBlue: "#5584b1",
             brightGreen: "#18Ed93",
+            brightRed: "#e53a40",
+            brightYellow: "#dedc12",
+            brightMagenta: "#9b7baf",
+            brightCyan: "#6da9bc",
         },
         customGlyphs: true,
         rescaleOverlappingGlyphs: true,
         allowProposedApi: true,
-        letterSpacing: 0,
-        screenReaderMode: true,
         //overviewRulerWidth: 30,
         // windowsPty: {
         //     backend: 'conpty',
@@ -59,20 +69,9 @@ terminal.loadAddon(new CanvasAddon.CanvasAddon());
 unicode11Addon = new Unicode11Addon.Unicode11Addon();
 terminal.loadAddon(unicode11Addon);
 terminal.unicode.activeVersion = '11';
-terminal.register({
-    wcwidth: (character) => {
-        const code = character.charCodeAt(0);
-        if (code == 0x1F525) return 2;  // Fire emoji
-        // Handle powerline symbols (usually should be width 1)
-        if (code >= 0xE0A0 && code <= 0xE0D4) return 1;
-        // Handle other specific unicode ranges
-        if (code >= 0x1100 && code <= 0x11FF) return 2;  // Hangul Jamo
-        if (code >= 0x3000 && code <= 0x30FF) return 2;  // CJK Symbols and Japanese
-        if (code >= 0x4E00 && code <= 0x9FFF) return 2;  // CJK Unified Ideographs
-        // Default to system wcwidth
-        return null;
-    }
-});
+
+UnicodeGraphemesAddon = new UnicodeGraphemesAddon.UnicodeGraphemesAddon();
+terminal.loadAddon(UnicodeGraphemesAddon);
 
 const fitAddon = new FitAddon.FitAddon();
 terminal.loadAddon(fitAddon);
@@ -83,14 +82,6 @@ terminal.onTitleChange((title) => {
     document.getElementById('commandInfo').innerText = title;
 })
 
-/*terminal.onResize((evt) => {
-    const terminal_size = {
-        Width: evt.cols,
-        Height: evt.rows,
-    };
-    console.log(terminal_size);
-})*/
-
 terminal.onSelectionChange(() => {
     const selectionText = terminal.getSelection();
     if (selectionText) {
@@ -117,12 +108,6 @@ function autoFit(scroll=true) {
     if (scroll) terminal.scrollToBottom();
 }
 
-function getTokenParam() {
-    const urlParams = new URLSearchParams(window.location.search);
-    return urlParams.get('token') ? `?token=${urlParams.get('token')}` : '';
-}
-const urlToken = getTokenParam();
-
 
 document.getElementById('launchForm').addEventListener('submit', async (event) => {
     event.preventDefault();
@@ -131,7 +116,7 @@ document.getElementById('launchForm').addEventListener('submit', async (event) =
     fitAddon.fit();
     terminal.clear();
     try {
-        const response = await fetch(`/run_command${urlToken}`, {
+        const response = await fetch(`/commands`, {
             method: 'POST',
             headers: {
                 'Content-Type': 'application/json'
@@ -142,7 +127,6 @@ document.getElementById('launchForm').addEventListener('submit', async (event) =
             throw new Error('Failed to launch command');
         }
         const data = await response.json();
-        //await new Promise(r => setTimeout(r, 300));// not ok
         viewOutput(data.command_id);
         fetchCommands();
         commandInput.focus()
@@ -154,12 +138,14 @@ document.getElementById('launchForm').addEventListener('submit', async (event) =
 
 async function fetchCommands(hide=false) {
     try {
-        const response = await fetch(`/commands${urlToken}`);
+        const response = await fetch(`/commands`);
         if (!response.ok) {
             document.getElementById('dimmer').style.display = 'block';
             return;
         }
-        const commands = await response.json();
+        // Adapt to the new result structure:
+        const data = await response.json();
+        const commands = data.commands;
         commands.sort((a, b) => new Date(b.start_time) - new Date(a.start_time));
         const commandsTbody = document.getElementById('commands');
         commandsTbody.innerHTML = '';
@@ -262,15 +248,15 @@ async function viewOutput(command_id) {
     outputPercentage.innerText = '100%';
     adjustOutputHeight();
     currentCommandId = command_id;
-    nextOutputLink = `/command_output/${command_id}${urlToken}`;
+    nextOutputLink = `/commands/${command_id}/output`;
     clearInterval(outputInterval);
     terminal.clear();
     terminal.reset();
     fullOutput = '';
     try {
-        const response = await fetch(`/command_status/${command_id}${urlToken}`);
+        const response = await fetch(`/commands/${command_id}`);
         if (!response.ok) {
-            return;
+            outputInterval = setInterval(() => fetchOutput(nextOutputLink), 500);
         }
         const data = await response.json();
         const commandInfo = document.getElementById('commandInfo');
@@ -299,7 +285,7 @@ async function viewOutput(command_id) {
 async function openPopup(command_id, event) {
     event.stopPropagation();
     event.stopImmediatePropagation();
-    const popupUrl = `/popup/${command_id}${urlToken}`;
+    const popupUrl = `/commands/${command_id}/popup`;
     window.open(popupUrl, '_blank', 'width=1000,height=600');
 }
 
@@ -307,7 +293,7 @@ async function relaunchCommand(command_id, event) {
     event.stopPropagation();
     event.stopImmediatePropagation();
     try {
-        const response = await fetch(`/command_status/${command_id}${urlToken}`);
+        const response = await fetch(`/commands/${command_id}`);
         if (!response.ok) {
             throw new Error('Failed to fetch command status');
         }
@@ -318,7 +304,7 @@ async function relaunchCommand(command_id, event) {
         }
         fitAddon.fit();
         terminal.clear();
-        const relaunchResponse = await fetch(`/run_command${urlToken}`, {
+        const relaunchResponse = await fetch(`/commands`, {
             method: 'POST',
             headers: {
                 'Content-Type': 'application/json'
@@ -346,8 +332,8 @@ async function stopCommand(command_id, event) {
     event.stopPropagation();
     event.stopImmediatePropagation();
     try {
-        const response = await fetch(`/stop_command/${command_id}${urlToken}`, {
-            method: 'POST'
+        const response = await fetch(`/commands/${command_id}/stop`, {
+            method: 'PATCH'
         });
         if (!response.ok) {
             throw new Error('Failed to stop command');
@@ -428,7 +414,7 @@ function sliderUpdateOutput()
 {
     const percentage = slider.value/10;
     outputLength = Math.floor((fullOutput.length * percentage) / 100);
-    const limitedOutput = fullOutput.slice(0, outputLength);
+    limitedOutput = fullOutput.slice(0, outputLength);
     terminal.clear();
     terminal.reset();
     terminal.write(limitedOutput);
